﻿using JewelryStoreContracts.BindingModels;
using JewelryStoreContracts.SearchModels;
using JewelryStoreContracts.StoragesContracts;
using JewelryStoreContracts.ViewModels;
using JewelryStoreDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace JewelryStoreDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new JewelryStoreDatabase();
            return context.Orders
            .Select(x => AccessJewelStorage(x.GetViewModel, context))
            .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new JewelryStoreDatabase();
            return context.Orders
            .Where(x => x.Id == model.Id)
            .Select(x => AccessJewelStorage(x.GetViewModel, context))
            .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new JewelryStoreDatabase();
            return AccessJewelStorage(context.Orders
            .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
            ?.GetViewModel, context);
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new JewelryStoreDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return AccessJewelStorage(newOrder.GetViewModel, context);
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new JewelryStoreDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return AccessJewelStorage(order.GetViewModel, context);
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new JewelryStoreDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return AccessJewelStorage(element.GetViewModel, context);
            }
            return null;
        }
        static OrderViewModel AccessJewelStorage(OrderViewModel model, JewelryStoreDatabase context)
        {
            if (model == null) return model;
            string? jewelName = context.Jewels.FirstOrDefault(x => x.Id == model.JewelId)?.JewelName;
            if (jewelName != null) model.JewelName = jewelName;
            return model;
        }
    }
}
